package cd.casic.ci.common.pipeline.container;

import cd.casic.ci.common.pipeline.IModelTemplate;
import cd.casic.ci.common.pipeline.option.StageControlOption;
import cd.casic.ci.common.pipeline.pojo.StagePauseCheck;
import cd.casic.ci.common.pipeline.pojo.time.BuildRecordTimeCost;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.List;
import java.util.Map;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.common.pipeline.container
 * @Project：ops-pro
 * @name：Stage
 * @Date：2025/03/25 16:55
 * @Filename：Stage
 * @description：Todo
 */
@Data
@NoArgsConstructor
@Schema(title = "流水线模型-阶段")
public class Stage implements IModelTemplate {
    @Schema(title = "容器集合", required = true)
    private List<Container> containers;
    @Schema(title = "阶段ID (系统标识，用户不可编辑)", required = false)
    private String id;
    @Schema(title = "阶段名称", required = true)
    private String name;
    @Schema(title = "阶段ID (用户可编辑)", required = false)
    private String stageIdForUser;
    @Schema(title = "阶段标签", required = false, readOnly = true)
    private List<String> tag;
    @Schema(title = "阶段状态", required = false, readOnly = true)
    private String status;
    @Schema(title = "阶段启动时间", required = false, readOnly = true)
    @Deprecated
    private Long startEpoch;
    @Schema(title = "容器运行时间", required = false, readOnly = true)
    @Deprecated
    private Long elapsed;
    @Schema(title = "用户自定义环境变量", required = false)
    private Map<String, String> customBuildEnv;
    @Schema(title = "是否启用容器失败快速终止阶段", required = false)
    private Boolean fastKill;
    @Schema(title = "标识是否为FinallyStage，每个Model只能包含一个FinallyStage，并且处于最后位置", required = false)
    private Boolean finallyStage;
    @Schema(title = "当前Stage是否能重试", required = false)
    private Boolean canRetry;
    @Schema(title = "流程控制选项", required = true)
    private StageControlOption stageControlOption;
    @Schema(title = "stage准入配置", required = false)
    private StagePauseCheck checkIn;
    @Schema(title = "stage准出配置", required = false)
    private StagePauseCheck checkOut;
    @Schema(title = "步骤运行次数", required = false, readOnly = true)
    private Integer executeCount;
    @Schema(title = "各项耗时", required = true)
    private BuildRecordTimeCost timeCost;

    public void resetBuildOption(Boolean init) {
        if (init != null && init) {
            status = null;
            startEpoch = null;
            elapsed = null;
        }
        if (checkIn != null) {
            checkIn.fixReviewGroups(init != null && init);
        }
        if (checkOut != null) {
            checkOut.fixReviewGroups(init != null && init);
        }
        if (stageControlOption != null && stageControlOption.getManualTrigger() && checkIn == null) {
            checkIn = StagePauseCheck.convertControlOption(stageControlOption);
        }
        if (finallyStage !=null && finallyStage){
            canRetry = false;
        }
    }

    public Container getContainer(String vmSeqId) {
        if (containers != null) {
            for (Container container : containers) {
                Container foundContainer = container.getContainerById(vmSeqId);
                if (foundContainer != null) {
                    return foundContainer;
                }
            }
        }
        return null;
    }

    public void transformCompatibility() {
        if (containers != null) {
            for (Container container : containers) {
                container.transformCompatibility();
            }
        }
    }

    public Boolean stageEnabled() {
        return stageControlOption != null ? stageControlOption.getEnable() : true;
    }

    public boolean isFinally() {
        return finallyStage != null && finallyStage;
    }

    @Override
    public String getTemplate() {
        return null;
    }

    @Override
    public String getRef() {
        return null;
    }

    @Override
    public Map<String, String> getVariables() {
        return null;
    }

    @Override
    public void setTemplate(String template) {

    }

    @Override
    public void setRef(String ref) {

    }

    @Override
    public void setVariables(Map<String, String> variables) {

    }
}
