package cd.casic.ci.common.pipeline.enums;

import cd.casic.ci.common.pipeline.pojo.element.trigger.*;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.Getter;
import org.slf4j.LoggerFactory;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.common.pipeline.enums
 * @Project：ops-pro
 * @name：StartType
 * @Date：2025/03/25 17:23
 * @Filename：StartType
 * @description：Todo
 */
@Getter
@AllArgsConstructor
public enum StartType {
    MANUAL,
    TIME_TRIGGER,
    WEB_HOOK,
    SERVICE,
    PIPELINE,
    REMOTE;

    public static String toReadableString(String type, ChannelCode channelCode, String language) {
        String[] params = null;
        String name = switch (type) {
            case "MANUAL" -> MANUAL.name();
            case "TIME_TRIGGER" -> TIME_TRIGGER.name();
            case "WEB_HOOK" -> WEB_HOOK.name();
            case "REMOTE" -> REMOTE.name();
            case "SERVICE" -> {
                if (channelCode != null) {
                    if (channelCode == ChannelCode.BS) {
                        yield SERVICE.name() + "_BS";
                    } else {
                        params = new String[]{channelCode.name()};
                        yield SERVICE.name() + "_CHANNEL";
                    }
                } else {
                    yield SERVICE.name() + "_NOT_CHANNEL";
                }
            }
            case "PIPELINE" -> PIPELINE.name();
            default -> "";
        };

        return name.isBlank() ? name : MessageFormat.format("START_TYPE_" + name, language, params);
    }

    public static StartType toStartType(String type) {
        for (StartType startType : values()) {
            if (type.equalsIgnoreCase(startType.name())) {
                return startType;
            }
        }
        LoggerFactory.getLogger(StartType.class).warn("Unknown start type(" + type + ")");
        return MANUAL;
    }

    public static List<IdValue> getStartTypeMap(String language) {
        List<IdValue> result = new ArrayList<>();
        for (StartType startType : values()) {
            result.add(new IdValue(startType.name(), toReadableString(startType.name(), null, language)));
        }
        return result;
    }

    public static String transform(String startType, String webhookType) {
        return switch (startType) {
            case "MANUAL" -> ManualTriggerElement.classType;
            case "TIME_TRIGGER" -> TimerTriggerElement.classType;
            case "WEB_HOOK" -> {
                yield switch (webhookType) {
                    case "SVN" -> CodeSVNWebHookTriggerElement.classType;
                    case "GIT" -> CodeGitWebHookTriggerElement.classType;
                    case "GITLAB" -> CodeGitlabWebHookTriggerElement.classType;
                    case "GITHUB" -> CodeGithubWebHookTriggerElement.classType;
                    case "TGIT" -> CodeTGitWebHookTriggerElement.classType;
                    default -> RemoteTriggerElement.classType;
                };
            }
            default -> RemoteTriggerElement.classType;
        };
    }

    @Data
    @AllArgsConstructor
    static class IdValue {
        private String id;
        private String value;
    }
}
