package cd.casic.ci.common.pipeline.pojo.element.agent;

import cd.casic.ci.common.pipeline.enums.CodePullStrategy;
import cd.casic.ci.common.pipeline.enums.RepositoryType;
import cd.casic.ci.common.pipeline.pojo.element.Element;
import cd.casic.ci.common.pipeline.pojo.git.GitPullMode;
import cn.hutool.json.JSONUtil;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.NoArgsConstructor;

import java.util.Map;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.common.pipeline.pojo.element.agent
 * @Project：ops-pro
 * @name：CodeGitlabElement
 * @Date：2025/03/26 16:34
 * @Filename：CodeGitlabElement
 * @description：Todo
 */
@NoArgsConstructor
@Schema(title = "拉取Gitlab仓库代码", description = CodeGitlabElement.classType)
public class CodeGitlabElement extends Element {
    public static final String classType = "CODE_GITLAB";
    public static final String modeType = "mode.type";
    public static final String modeValue = "mode.value";

    @Schema(title = "任务名称", required = true)
    private String name = "";
    @Schema(title = "id", required = false)
    private String id;
    @Schema(title = "状态", required = false)
    private String status;
    @Schema(title = "代码库哈希ID", required = true)
    private String repositoryHashId;
    @Schema(title = "分支名称", required = false)
    private String branchName;
    @Schema(title = "revision 用于强制指定commitId", required = false)
    private String revision;
    @Schema(title = "checkout 策略", required = false)
    private CodePullStrategy strategy = CodePullStrategy.INCREMENT_UPDATE;
    @Schema(title = "代码存放路径", required = false)
    private String path;
    @Schema(title = "启动Submodule", required = false)
    private Boolean enableSubmodule = true;
    @Schema(title = "Git指定拉取方式", required = false)
    private GitPullMode gitPullMode;
    @Schema(title = "新版的gitlab原子的类型")
    private RepositoryType repositoryType;
    @Schema(title = "新版的gitlab代码库名")
    private String repositoryName;

    @Override
    public Map<String, Object> genTaskParams() {
        Map<String, Object> paramMap = JSONUtil.toBean(JSONUtil.toJsonStr(this), Map.class);
        if (gitPullMode != null) {
            paramMap.put(modeType, gitPullMode.getType().name());
            paramMap.put(modeValue, gitPullMode.getValue());
        }
        return paramMap;
    }

    @Override
    public String getClassType() {
        return classType;
    }
}
