package cd.casic.ci.common.pipeline.pojo.element.trigger;

import cd.casic.ci.common.pipeline.enums.StartType;
import cd.casic.ci.common.pipeline.enums.TriggerRepositoryType;
import cd.casic.ci.common.pipeline.pojo.element.ElementProp;
import cd.casic.ci.common.pipeline.pojo.element.trigger.enums.CodeEventType;
import cd.casic.ci.common.pipeline.pojo.element.trigger.enums.PathFilterType;
import cd.casic.ci.common.pipeline.utils.TriggerElementPropUtils;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.experimental.Accessors;

import java.util.List;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.common.pipeline.pojo.element.trigger
 * @Project：ops-pro
 * @name：CodeGitWebHookTriggerElement
 * @Date：2025/03/27 10:20
 * @Filename：CodeGitWebHookTriggerElement
 * @description：Todo
 */
@Accessors(chain = true)
@Schema(title = "Git事件触发", description = CodeGitWebHookTriggerElement.classType)
public class CodeGitWebHookTriggerElement extends WebHookTriggerElement {
    public static final String classType = "codeGitWebHookTrigger";
    public static final String MERGE_ACTION_OPEN = "open";
    public static final String MERGE_ACTION_CLOSE = "close";
    public static final String MERGE_ACTION_REOPEN = "reopen";
    public static final String MERGE_ACTION_PUSH_UPDATE = "push-update";
    public static final String MERGE_ACTION_MERGE = "merge";
    public static final String PUSH_ACTION_CREATE_BRANCH = "new-branch";
    public static final String PUSH_ACTION_PUSH_FILE = "push-file";

    @Schema(title = "任务名称", required = true)
    private String name = "Git事件触发";

    @Schema(title = "id", required = false)
    private String id;

    @Schema(title = "状态", required = false)
    private String status;

    @Schema(title = "仓库ID", required = true)
    private String repositoryHashId;

    @Schema(title = "分支名称", required = false)
    private String branchName;

    @Schema(title = "用于排除的分支名", required = false)
    private String excludeBranchName;

    @Schema(title = "路径过滤类型", required = true)
    private PathFilterType pathFilterType = PathFilterType.NamePrefixFilter;

    @Schema(title = "用于包含的路径", required = false)
    private String includePaths;

    @Schema(title = "用于排除的路径", required = false)
    private String excludePaths;

    @Schema(title = "用户白名单", required = false)
    private List<String> includeUsers;

    @Schema(title = "用于排除的user id", required = false)
    private List<String> excludeUsers;

    @Schema(title = "事件类型", required = false)
    private CodeEventType eventType;

    @Schema(title = "是否为block", required = false)
    private Boolean block;

    @Schema(title = "新版的git原子的类型")
    private TriggerRepositoryType repositoryType;

    @Schema(title = "新版的git代码库名")
    private String repositoryName;

    @Schema(title = "tag名称", required = false)
    private String tagName;

    @Schema(title = "用于排除的tag名称", required = false)
    private String excludeTagName;

    @Schema(title = "tag从哪条分支创建", required = false)
    private String fromBranches;

    @Schema(title = "用于排除的源分支名称", required = false)
    private String excludeSourceBranchName;

    @Schema(title = "用于包含的源分支名称", required = false)
    private String includeSourceBranchName;

    @Schema(title = "webhook队列", required = false)
    private Boolean webhookQueue = false;

    @Schema(title = "code review 状态", required = false)
    private List<String> includeCrState;

    @Schema(title = "code review 类型", required = false)
    private List<String> includeCrTypes;

    @Schema(title = "code note comment", required = false)
    private String includeNoteComment;

    @Schema(title = "code note 类型", required = false)
    private List<String> includeNoteTypes;

    @Schema(title = "是否启用回写")
    private Boolean enableCheck = true;

    @Schema(title = "issue事件action")
    private List<String> includeIssueAction;

    @Schema(title = "mr事件action")
    private List<String> includeMrAction;

    @Schema(title = "push事件action")
    private List<String> includePushAction;

    @Schema(title = "是否启用第三方过滤")
    private Boolean enableThirdFilter = false;

    @Schema(title = "第三方应用地址")
    private String thirdUrl;

    @Schema(title = "第三方应用鉴权token")
    private String thirdSecretToken;

    @Schema(title = "跳过WIP")
    private Boolean skipWip = false;

    public CodeGitWebHookTriggerElement(String name, String id, String status) {
        super(name, id, status);
    }

    @Override
    public String getClassType() {
        return classType;
    }

    @Override
    public String findFirstTaskIdByStartType(StartType startType) {
        return StartType.WEB_HOOK.name().equals(startType.name()) ? this.id : super.findFirstTaskIdByStartType(startType);
    }

    @Override
    public List<ElementProp> triggerCondition() {
        List<ElementProp> props;
        switch (eventType) {
            case PUSH:
                props = List.of(
                        TriggerElementPropUtils.vuexInput("action", joinToString(includePushAction)),
                        TriggerElementPropUtils.vuexInput("branchName", branchName),
                        TriggerElementPropUtils.vuexInput("excludeBranchName", excludeBranchName),
                        TriggerElementPropUtils.vuexInput("includePaths", includePaths),
                        TriggerElementPropUtils.vuexInput("excludePaths", excludePaths),
                        TriggerElementPropUtils.staffInput("includeUsers", includeUsers),
                        TriggerElementPropUtils.staffInput("excludeUsers", excludeUsers)
                );
                break;
            case MERGE_REQUEST:
                props = List.of(
                        TriggerElementPropUtils.vuexInput("action", joinToString(includeMrAction)),
                        TriggerElementPropUtils.selector("skip-wip", List.of((skipWip != null ? skipWip.toString() : "false"))),
                        TriggerElementPropUtils.vuexInput("branchName", branchName),
                        TriggerElementPropUtils.vuexInput("excludeBranchName", excludeBranchName),
                        TriggerElementPropUtils.vuexInput("includeSourceBranchName", includeSourceBranchName),
                        TriggerElementPropUtils.vuexInput("includeSourceBranchName", includeSourceBranchName),
                        TriggerElementPropUtils.vuexInput("includePaths", includePaths),
                        TriggerElementPropUtils.vuexInput("excludePaths", excludePaths),
                        TriggerElementPropUtils.staffInput("includeUsers", includeUsers),
                        TriggerElementPropUtils.staffInput("excludeUsers", excludeUsers)
                );
                break;
            case MERGE_REQUEST_ACCEPT:
                props = List.of(
                        TriggerElementPropUtils.vuexInput("action", "merge"),
                        TriggerElementPropUtils.vuexInput("branchName", branchName),
                        TriggerElementPropUtils.vuexInput("excludeBranchName", excludeBranchName),
                        TriggerElementPropUtils.vuexInput("includeSourceBranchName", includeSourceBranchName),
                        TriggerElementPropUtils.vuexInput("includeSourceBranchName", includeSourceBranchName),
                        TriggerElementPropUtils.vuexInput("includePaths", includePaths),
                        TriggerElementPropUtils.vuexInput("excludePaths", excludePaths),
                        TriggerElementPropUtils.staffInput("includeUsers", includeUsers),
                        TriggerElementPropUtils.staffInput("excludeUsers", excludeUsers)
                );
                break;
            case TAG_PUSH:
                props = List.of(
                        TriggerElementPropUtils.vuexInput("tagName", tagName),
                        TriggerElementPropUtils.vuexInput("excludeTagName", excludeTagName),
                        TriggerElementPropUtils.vuexInput("fromBranches", fromBranches),
                        TriggerElementPropUtils.staffInput("includeUsers", includeUsers),
                        TriggerElementPropUtils.staffInput("excludeUsers", excludeUsers)
                );
                break;
            case REVIEW:
                props = List.of(
                        TriggerElementPropUtils.selector("includeCrState", includeCrState),
                        TriggerElementPropUtils.selector("includeCrTypes", includeCrTypes)
                );
                break;
            case ISSUES:
                props = List.of(
                        TriggerElementPropUtils.selector("includeIssueAction", includeIssueAction)
                );
                break;
            case NOTE:
                props = List.of(
                        TriggerElementPropUtils.selector("includeNoteTypes", includeNoteTypes),
                        TriggerElementPropUtils.vuexInput("includeNoteComment", includeNoteComment)
                );
                break;
            default:
                props = List.of();
        }
        return props.stream().filter(java.util.Objects::nonNull).toList();
    }
}
