package cd.casic.ci.common.pipeline.type;

import cd.casic.ci.common.pipeline.pojo.agent.ThirdPartyAgentEnvDispatchType;
import cd.casic.ci.common.pipeline.pojo.agent.ThirdPartyAgentIDDispatchType;
import cd.casic.ci.common.pipeline.type.docker.DockerDispatchType;
import cd.casic.ci.common.pipeline.type.kubernetes.KubernetesDispatchType;
import cd.casic.ci.common.pipeline.utils.EnvUtils;
import com.fasterxml.jackson.annotation.JsonSubTypes;
import com.fasterxml.jackson.annotation.JsonTypeInfo;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.Map;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.common.pipeline.type
 * @Project：ops-pro
 * @name：DispatchType
 * @Date：2025/03/27 10:45
 * @Filename：DispatchType
 * @description：Todo
 */

@JsonTypeInfo(use = JsonTypeInfo.Id.NAME, include = JsonTypeInfo.As.PROPERTY, property = "buildType", visible = false)
@JsonSubTypes({
        @JsonSubTypes.Type(value = DockerDispatchType.class, name = "DOCKER"),
        @JsonSubTypes.Type(value = KubernetesDispatchType.class, name = "KUBERNETES"),
        @JsonSubTypes.Type(value = ThirdPartyAgentIDDispatchType.class, name = "THIRD_PARTY_AGENT_ID"),
        @JsonSubTypes.Type(value = ThirdPartyAgentEnvDispatchType.class, name = "THIRD_PARTY_AGENT_ENV")
})
@Data
@AllArgsConstructor
@NoArgsConstructor
public abstract class DispatchType {
    private String value;
    private DispatchRouteKeySuffix routeKeySuffix;

    public void replaceVariable(Map<String, String> variables) {
        this.value = EnvUtils.parseEnv(value, variables);
        replaceField(variables);
    }

    public abstract BuildType buildType();

    protected abstract void replaceField(Map<String, String> variables);

    public abstract void cleanDataBeforeSave();
}
