package cd.casic.ci.log.dal.pojo.meta;

import org.apache.logging.log4j.util.Strings;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Callable;

public class Ansi {
    private static char FIRST_ESC_CHAR = 27;
    private static char SECOND_ESC_CHAR = '[';
    private static String DISABLE = Ansi.class.getName() + ".disable";
    private static Callable<Boolean> detector;
    private StringBuilder builder;

    private List<Integer> attributeOptions = new ArrayList<>(5);

    public void newDetector(Callable<Boolean> detector) {
        if (detector == null) {
            throw new IllegalArgumentException("detector is null");
        }
        this.detector = detector;
    }

    public boolean isDetected() {
        try {
            return detector.call();
        } catch (Exception e) {
            return true;
        }
    }

    private InheritableThreadLocal<Boolean> holder = new InheritableThreadLocal<Boolean>() {
        @Override
        public Boolean initialValue(){
            return isDetected();
        }
    };

    public Ansi bold() {
        return a(AnsiAttribute.INTENSITY_BOLD);
    }

    public Ansi a(AnsiAttribute attribute) {
        attributeOptions.add(attribute.value);
        return this;
    }

    public Ansi fgYellow() {
        return this.fg(AnsiColor.YELLOW);
    }

    public Ansi fgGreen() {
        return this.fg(AnsiColor.GREEN);
    }

    public Ansi fgRed() {
        return this.fg(AnsiColor.RED);
    }

    public Ansi fg(AnsiColor color) {
        attributeOptions.add(color.fg());
        return this;
    }

    public Ansi a(String value) {
        // 修复换行后ansi编码问题
        String strLF = innerAppendEscapeSequenceStr('m', attributeOptions);
        // 清空属性？？？
        flushAttributes();
        // 对value进行正则替换
        String fixLF = "";
        // 对 value 的空安全判断
        if (!Strings.isBlank(value))
            fixLF = value.replace("\n", "\n$strLF");
        builder.append(fixLF);
        return this;
    }

    private void flushAttributes() {
        if (attributeOptions.isEmpty()) {
            return;
        }
        if (attributeOptions.size() == 1 && attributeOptions.get(0) == 0) {
            builder.append(FIRST_ESC_CHAR);
            builder.append(SECOND_ESC_CHAR);
            builder.append('m');
        } else {
            // 内部添加分号
            innerAppendEscapeSequence('m', attributeOptions);
        }
        attributeOptions.clear();
    }

    private Ansi innerAppendEscapeSequence(char command, List<Integer> options) {
        builder.append(FIRST_ESC_CHAR);
        builder.append(SECOND_ESC_CHAR);
        int size = options.size();
        for (int i = 0;i < size;i++) {
            if (i != 0) {
                builder.append(';');
            }
            builder.append(options.get(i));
        }
        builder.append(command);
        return this;
    }

    private String innerAppendEscapeSequenceStr(char command,List<Integer> options) {
        StringBuilder str = new StringBuilder();
        str.append(FIRST_ESC_CHAR);
        str.append(SECOND_ESC_CHAR);
        int size = options.size();
        for (int i = 0; i < size;i++) {
            if (i != 0) {
                str.append(';');
            }
            str.append(options.get(i));
        }
        str.append(command);
        return str.toString();
    }

    public Ansi reset() {
        return a(AnsiAttribute.RESET);
    }

    @Override
    public String toString() {
        flushAttributes();
        return builder.toString();
    }
}
