package cd.casic.ci.log.service;

import cd.casic.ci.log.dal.pojo.EndPageQueryLogs;
import cd.casic.ci.log.dal.pojo.PageQueryLogs;
import cd.casic.ci.log.dal.pojo.QueryLogs;
import cd.casic.ci.log.log.LogStorageEvent;
import cd.casic.ci.log.dal.pojo.enums.LogType;
import cd.casic.ci.log.log.LogOriginEvent;
import cd.casic.ci.log.log.LogStatusEvent;
import cd.casic.ci.log.jmx.CreateIndexBean;
import cd.casic.ci.log.jmx.LogStorageBean;
import cd.casic.ci.log.dal.pojo.message.LogMessage;
import cd.casic.ci.log.dal.pojo.message.LogMessageWithLineNo;
import com.github.benmanes.caffeine.cache.Cache;
import com.github.benmanes.caffeine.cache.Caffeine;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.redisson.RedissonRedLock;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * @author by mianbin
 * @Classname LogServiceImpl
 * @Description TODO
 * @Date 2025/3/20 10:52
 */
@Slf4j
@Service
public class LogServiceImpl implements LogService {
    @Resource
    private IndexService indexService;
    @Resource
    private LogStatusService logStatusService;
    @Resource
    private LogTagService logTagService;
    @Resource
    private CreateIndexBean createIndexBean;
    @Resource
    private LogStorageBean logStorageBean;
    @Resource
    private BuildLogPrintService buildLogPrintService;
    @Resource
    private RedissonClient redissonClient;

    private static long LONG_SEARCH_TIME = 64000;
    private static long SHORT_SEARCH_TIME = 32000;
    private static long SEARCH_TIMEOUT_SECONDS = 60L;
    private static long SEARCH_FRAGMENT_SIZE = 100000;
    private static long INDEX_CACHE_MAX_SIZE = 100000L;
    private static long INDEX_CACHE_EXPIRE_MINUTES = 30L;
    private static long INDEX_LOCK_EXPIRE_SECONDS = 10L;
    private static long INDEX_STORAGE_WARN_MILLIS = 1000;
    private Cache<String, Boolean> indexCache = Caffeine.newBuilder()
            .maximumSize(INDEX_CACHE_MAX_SIZE).expireAfterAccess(INDEX_CACHE_EXPIRE_MINUTES, TimeUnit.MINUTES)
            .build();

    @Override
    public void addLogEvent(LogOriginEvent event) {
//        List<LogMessageWithLineNo> logMessage = addLineNo(event.getBuildId(),event.getLogs());

//        if (!logMessage.isEmpty()) {
//            buildLogPrintService.dispatchEvent(new LogStorageEvent(event.getBuildId(), logMessage));
//        }
    }

    @Override
    public void addBatchLogEvent(LogStorageEvent event) {
        long currentEpoch = System.currentTimeMillis();
        boolean success = false;
        try {
            // todo 提前准备索引
            prepareIndex(event.getBuildId());
            List<LogMessageWithLineNo> logMessages = event.getLogs();
            List<LogMessageWithLineNo> buf = new ArrayList<>();
            // 遍历日志信息
            for (LogMessageWithLineNo logMessage : logMessages) {
                buf.add(logMessage);
//                if (buf.size() == Constants.BULK_BUFFER_SIZE){
//                    // 添加多行日志
//                    if (doAddMultiLines(buf,event.getBuildId()) == 0){
//                        throw new Exception(
//                                "None of lines is inserted successfully to ES " +
//                                        "["+event.getBuildId()+"|"+event.getRetryTime()+"}]"
//                        );
//                    } else {
//                        buf.clear();
//                    }
//                }
            }
            if (!buf.isEmpty()) {
                if (doAddMultiLines(buf, event.getBuildId()) == 0) {
                    throw new Exception("None of lines is inserted successfully to ES" + "[" + event.getBuildId() + "|" + event.getRetryTime() + "}]");
                }
            }
            success = true;
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            long elapse = System.currentTimeMillis() - currentEpoch;
            logStorageBean.batchWrite(elapse, success);

            // #4265 当日志消息处理时间过长时打印消息内容
            if (elapse >= INDEX_STORAGE_WARN_MILLIS && !event.getLogs().isEmpty()) {
                log.warn("[" + event.getBuildId() + "] addBatchLogEvent spent too much time(" + elapse + ") with tag=" + event.getLogs().get(0).getTag() + "}");
            }
        }
    }

    private int doAddMultiLines(List<LogMessageWithLineNo> buf, String buildId) {
        return 0;
    }

    // todo 在ES里面创建index
    private void createIndex(String buildId, String index) {

    }

    // todo 给ES数据库存消息
    private boolean isExistIndex(String buildId, String index) {
        return true;
    }

    @Override
    public QueryLogs queryInitLogs(String buildId, boolean debug, LogType logType, String tag, String subTag, String jobId, int executeCount) {
        return doQueryInitLogs(buildId, debug, logType, tag, subTag, jobId, executeCount);
    }

    private QueryLogs doQueryInitLogs(String buildId, boolean debug, LogType logType, String tag, String subTag, String jobId, int executeCount) {
        // todo 从 ES 数据库里面取出数据
        return null;
    }

    @Override
    public QueryLogs queryLogsBetweenLines(String buildId, int num, boolean fromStart, long start, long end, boolean debug, LogType logType, String tag, String subTag, String jobId, int executeCount) {
        return null;
    }

    @Override
    public QueryLogs queryLogsAfterLine(String buildId, long start, boolean debug, LogType logType, String tag, String subTag, String jobId, int executeCount) {
        return null;
    }

    @Override
    public PageQueryLogs queryInitLogsPage(String buildId, boolean debug, LogType logType, String tag, String subTag, String jobId, int executeCount, int page, int pageSize) {
        return null;
    }

    @Override
    public QueryLogs queryLogsBeforeLine(String buildId, long end, int size, boolean debug, LogType logType, String tag, String subTag, String jobId, int executeCount) {
        return null;
    }

    @Override
    public EndPageQueryLogs getEndLogsPage(String pipelineId, String buildId, boolean debug, LogType logType, String tag, String subTag, String jobId, int executeCount, int size) {
        return null;
    }

    @Override
    public QueryLogs getBottomLogs(String pipelineId, String buildId, boolean debug, LogType logType, String tag, String subTag, String jobId, int executeCount, int size) {
        return null;
    }

    @Override
    public void updateLogStatus(LogStatusEvent event) {
//        logStatusService.finish(event.getBuildId(),event.getTag(),event.getSubTag(),
//                event.getJobId(),event.getExecuteCount(),event.getLogStorageMode(),event.getFinished());
    }

    private List<LogMessageWithLineNo> addLineNo(String buildId, List<LogMessage> logMessages) {
        long lineNum = indexService.getAndAddLineNum(buildId, logMessages.size());

        if (lineNum == -1) {
            log.error("Got null logIndex from indexService, buildId: " + buildId);
            return new ArrayList<>();
        }
        // 不为空的话
        long startLineNum = lineNum;
        List<LogMessageWithLineNo> res = new ArrayList<>();
        // 遍历 logMessages
        for (LogMessage logMessage : logMessages) {
            // 设置 timestamp
            long timestamp = logMessage.getTimestamp();
            if (timestamp == 0L) {
                timestamp = System.currentTimeMillis();
            }
            // 如果 tag 不为空
            if (!StringUtils.isEmpty((logMessage.getSubTag()))) {
                logTagService.saveSubTag(buildId, logMessage.getTag(), logMessage.getSubTag());
            }
            //todo 这个传参先null，后面再弄
            res.add(new LogMessageWithLineNo(logMessage.getTag(), logMessage.getSubTag(),
                    logMessage.getJobId(), logMessage.getMessage(),
                    timestamp, logMessage.getLogType(), startLineNum++, logMessage.getExecuteCount(), null, null)
            );
        }
        return res;
    }

    private boolean prepareIndex(String buildId) {
        String index = indexService.getIndexName(buildId);
        if (!checkIndexCreate(buildId, index)) {
            createIndex(buildId, index);
            indexCache.put(index, true);
            return true;
        }
        return true;
    }

    private boolean checkIndexCreate(String buildId, String index) {
        if (indexCache.getIfPresent(index) == true) {
            return true;
        }
        RLock lock = redissonClient.getLock("LOG:index:create:lock:key:$index");
        RedissonRedLock redissonRedLock = new RedissonRedLock(lock);
        try {
            redissonRedLock.lock();
            if (indexCache.getIfPresent(index) == true) {
                return true;
            }
            // Check from ES
            if (isExistIndex(buildId, index)) {
                log.info("[" + buildId + "|" + index + "] the index is already created");
                indexCache.put(index, true);
                return true;
            }
            return false;
        } finally {
            redissonRedLock.unlock();
        }
    }
}
