package cd.casic.ci.process.engine.scheduler;

import cd.casic.ci.process.engine.enums.ContextStateEnum;
import cd.casic.ci.process.engine.executor.PipelineExecutor;
import cd.casic.ci.process.engine.scheduler.config.PipelineSchedulerConfig;
import cd.casic.ci.process.engine.scheduler.dateObject.PipelineSchedulingProperties;
import cd.casic.ci.process.engine.scheduler.service.impl.PipelineSchedulingPropertiesServiceImpl;
import jakarta.annotation.PostConstruct;
import jakarta.annotation.Resource;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @author HopeLi
 * @version v1.0
 * @ClassName PipelineSchedulingBootstrapper
 * @Date: 2025/5/26 17:28
 * @Description:
 */
@Component
@RequiredArgsConstructor
public class PipelineSchedulingBootstrapper {
    @Resource
    private PipelineSchedulingPropertiesServiceImpl taskService;

    @Resource
    private PipelineSchedulerConfig taskScheduler;

    @Resource
    private PipelineExecutor pipelineExecutor;

    @PostConstruct
    public void init() {
        List<PipelineSchedulingProperties> tasks = taskService.getAllTasks();
        for (PipelineSchedulingProperties task : tasks) {
            if (ContextStateEnum.RUNNING.getCode().equals(Integer.parseInt(task.getStatus()))) {
                taskScheduler.addTask(task.getPipelineId(), ()->{
                    pipelineExecutor.execute(task.getPipelineId());
                }, task.getCronExpression());
            }
        }
    }
}
