package cd.casic.ci.log.scm.api;


import cd.casic.ci.log.scm.dal.pojo.github.GithubBranch;
import cd.casic.ci.log.scm.dal.pojo.github.GithubTag;
import cd.casic.ci.log.scm.dal.pojo.github.GithubToken;
import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.ci.log.scm.enums.GitHubTokenType;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@RestController
@Tag(name = "服务-github相关")
@RequestMapping("/service/github")
public interface ServiceGithubResource {

    @Operation(summary = "获取github代码库accessToken")
    @GetMapping("/accessToken")
    CommonResult<GithubToken> getAccessToken(
            @Parameter(name = "userId", description = "用户ID", required = true)
            @RequestParam(name = "userId")
            String userId,
            @Parameter(description = "token 类型", required = false)
            @RequestParam(name = "tokenType",defaultValue = "GITHUB_APP")
            GitHubTokenType gitHubTokenType
            );

    @Operation(summary = "获取github指定分支")
    @GetMapping("/getGithubBranch")
    CommonResult<GithubBranch> getGithubBranch(
            @Parameter(name = "accessToken", required = true)
            @RequestParam(name = "accessToken")
            String accessToken,
            @Parameter(name = "projectName", required = true)
            @RequestParam(name = "projectName")
            String projectName,
            @Parameter(name = "branch", required = false)
            @RequestParam(name = "branch")
            String branch);

    @Operation(summary = "获取github指定tag")
    @GetMapping("/getGithubTag")
    CommonResult<GithubTag> getGithubTag(
            @Parameter(name = "accessToken", required = true)
            @RequestParam(name = "accessToken")
            String accessToken,
            @Parameter(name = "projectName", required = true)
            @RequestParam(name = "projectName")
            String projectName,
            @Parameter(name = "tag", required = true)
            @RequestParam(name = "tag")
            String tag
    );
}
