package cd.casic.pipeline.store.api.common;


import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.pipeline.store.pojo.common.QueryComponentPkgEnvInfoParam;
import cd.casic.pipeline.store.pojo.common.enums.ReleaseTypeEnum;
import cd.casic.pipeline.store.pojo.common.enums.StoreTypeEnum;
import cd.casic.pipeline.store.pojo.common.publication.StorePkgEnvInfo;
import cd.casic.pipeline.store.pojo.common.publication.StorePkgInfoUpdateRequest;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import java.util.List;

import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID;
import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID_DEFAULT_VALUE;


@Tag(name = "SERVICE_STORE_ARCHIVE", description = "研发商店-组件归档")
@RestController
@RequestMapping("/service/store/archives")
public interface ServiceStoreArchiveResource {

    @Operation(summary = "校验用户上传的组件包是否合法")
    @GetMapping("/types/{storeType}/codes/{storeCode}/versions/{version}/package/verify")
    CommonResult<Boolean> verifyComponentPackage(
            @Parameter(description = "用户ID", required = true, example = AUTH_HEADER_USER_ID_DEFAULT_VALUE)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "组件类型", required = true)
            @PathVariable("storeType")
            StoreTypeEnum storeType,
            @Parameter(description = "组件代码", required = true)
            @PathVariable("storeCode")
            String storeCode,
            @Parameter(description = "版本号", required = true)
            @PathVariable("version")
            String version,
            @Parameter(description = "发布类型", required = false)
            @RequestParam("releaseType")
            ReleaseTypeEnum releaseType
    );

    @Operation(summary = "获取组件包环境信息")
    @GetMapping("/types/{storeType}/codes/{storeCode}/versions/{version}/pkg/env/info/get")
    CommonResult<List<StorePkgEnvInfo>> getComponentPkgEnvInfo(
            @Parameter(description = "用户ID", required = true, example = AUTH_HEADER_USER_ID_DEFAULT_VALUE)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "组件类型", required = true)
            @PathVariable("storeType")
            StoreTypeEnum storeType,
            @Parameter(description = "组件代码", required = true)
            @PathVariable("storeCode")
            String storeCode,
            @Parameter(description = "版本号", required = true)
            @PathVariable("version")
            String version,
            @Parameter(description = "操作系统名称", required = false)
            @RequestParam("osName")
            String osName,
            @Parameter(description = "操作系统架构", required = false)
            @RequestParam("osArch")
            String osArch
    );

    @Operation(summary = "根据配置文件内容获取组件包环境信息")
    @PostMapping("/types/{storeType}/codes/{storeCode}/versions/{version}/pkg/env/info/get")
    CommonResult<List<StorePkgEnvInfo>> getComponentPkgEnvInfo(
            @Parameter(description = "用户ID", required = true, example = AUTH_HEADER_USER_ID_DEFAULT_VALUE)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "组件类型", required = true)
            @PathVariable("storeType")
            StoreTypeEnum storeType,
            @Parameter(description = "组件代码", required = true)
            @PathVariable("storeCode")
            String storeCode,
            @Parameter(description = "版本号", required = true)
            @PathVariable("version")
            String version,
            QueryComponentPkgEnvInfoParam queryComponentPkgEnvInfoParam
    );

    @Operation(summary = "更新组件执行包相关信息")
    @PutMapping("/component/pkg/info/update")
    CommonResult<Boolean> updateComponentPkgInfo(
            @Parameter(description = "用户ID", required = true, example = AUTH_HEADER_USER_ID_DEFAULT_VALUE)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            StorePkgInfoUpdateRequest storePkgInfoUpdateRequest
    );
}