package cd.casic.pipeline.store.api.common;

import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.pipeline.store.pojo.common.StoreErrorCodeInfo;
import cd.casic.pipeline.store.pojo.common.enums.StoreTypeEnum;
import cd.casic.pipeline.store.pojo.common.statistic.StoreStatistic;
import cd.casic.pipeline.store.pojo.common.statistic.StoreStatisticTrendData;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID;

@Tag(name = "USER_STORE_STATISTIC", description = "研发商店-统计")
@RestController
@RequestMapping("/user/store/statistic")
public interface UserStoreStatisticResource {

    @Operation(summary = "获取store组件基本统计数据信息")
    @GetMapping("/types/{storeType}/codes/{storeCode}")
    CommonResult<StoreStatistic> getStatisticByCode(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "组件类型", required = true)
            @PathVariable("storeType")
            StoreTypeEnum storeType,
            @Parameter(description = "插件标识", required = true)
            @PathVariable("storeCode")
            String storeCode
    );

    @Operation(summary = "获取store组件统计趋势数据信息")
    @GetMapping("/types/{storeType}/codes/{storeCode}/trend/data")
    CommonResult<StoreStatisticTrendData> getStatisticTrendDataByCode(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "组件类型", required = true)
            @PathVariable("storeType")
            StoreTypeEnum storeType,
            @Parameter(description = "插件标识", required = true)
            @PathVariable("storeCode")
            String storeCode,
            @Parameter(description = "查询开始时间，格式yyyy-MM-dd HH:mm:ss", required = true)
            @RequestParam("startTime")
            String startTime,
            @Parameter(description = "查询结束时间，格式yyyy-MM-dd HH:mm:ss", required = true)
            @RequestParam("endTime")
            String endTime
    );

    @Operation(summary = "获取store组件错误码信息")
    @GetMapping("/types/{storeType}/codes/{storeCode}/errorCode")
    CommonResult<StoreErrorCodeInfo> getStoreErrorCodeInfo(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "组件类型", required = true)
            @PathVariable("storeType")
            StoreTypeEnum storeType,
            @Parameter(description = "组件标识", required = true)
            @PathVariable("storeCode")
            String storeCode
    );
}