package cd.casic.pipeline.store.pojo.common.enums;

import lombok.Getter;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

@Getter
public enum StoreStatusEnum {
    INIT(true), // 初始化
    COMMITTING(true), // 提交中
    BUILDING(true), // 构建中
    BUILD_FAIL(true), // 构建失败
    CHECKING(true), // 验证中
    CHECK_FAIL(true), // 验证失败
    TESTING(true), // 测试中
    EDITING(true), // 填写信息中
    AUDITING(true), // 审核中
    AUDIT_REJECT(false), // 审核驳回
    RELEASED(false), // 已发布
    GROUNDING_SUSPENSION(false), // 上架中止
    UNDERCARRIAGING(false), // 下架中
    UNDERCARRIAGED(false), // 已下架
    TESTED(false); // 测试结束(仅分支测试使用)

    private final boolean isProcessing;

    StoreStatusEnum(boolean isProcessing) {
        this.isProcessing = isProcessing;
    }

    public static List<String> getProcessingStatusList() {
        return Arrays.stream(values())
                .filter(StoreStatusEnum::isProcessing)
                .map(Enum::name)
                .collect(Collectors.toList());
    }

    public static List<String> getTestStatusList() {
        return Arrays.asList(
                TESTING.name(),
                EDITING.name(),
                AUDITING.name()
        );
    }
}